#!/bin/sh

#######################################
# Utility functions
#######################################

usage()
{
	echo "Usage: $0 <create|destroy|debug|reset> -a <storage account> -c <storage class> -r <storage cluster role> [-n <namespace> ] [-v]"
}

fill_templates()
{
	local namespace="$1"
	local storage_account="$2"
	local storage_class="$3"
	local storage_cluster_role="$4"
	local pvc="$5"
	local pod="$6"

	if [ ! -d ./templates ]; then
		echo "templates directory is missing"
		exit 1
	fi

	rm -f rendered 2>/dev/null
	mkdir rendered 2>/dev/null

	for f in $(ls templates); do
		sed	-e "s|@@STORAGE_ACCOUNT@@|${storage_account}|g" \
			-e "s|@@STORAGE_CLUSTER_ROLE@@|${storage_cluster_role}|g" \
			-e "s|@@STORAGE_CLASS@@|${storage_class}|g" \
			-e "s|@@PVC@@|${pvc}|g" \
			-e "s|@@POD@@|${pod}|g" \
			"templates/$f" > "rendered/$f"
	
	done
}

get_pv()
{
	oc get pv|grep '^pvc'|head -n 1|awk '{ print $1 }'
}

get_pvc()
{
	oc get pvc --namespace="$1"|grep -v NAME|grep "$2"|awk '{ print $1 }'
}


#######################################
# Delete operations
#######################################

# delete pod
delete_pod()
{
	local namespace="$1"
	local pod="$2"

	oc delete pod "${pod}" --namespace="${namespace}"
	return $?
}

# delete persistent volume claim
delete_pvc()
{
	local namespace="$1"
	local pvc="$2"

	oc delete pvc "${pvc}" --namespace="${namespace}"
	return $?
}

# delete persistent volume
delete_pv()
{
	local namespace="$1"
	local pvc="$1"

	local pvs="$(get_pv ${namespace} ${pvc})"
	if [ -z "${pvs}" ]; then
		return 1
	fi

	for pv in ${pvs}; do
		oc delete pv "${pv}" --namespace="${namespace}"
	done

	return $?
}

# delete storage class
delete_sc()
{
	local namespace="$1"
	local storage_class="$2"

	oc delete storageclass "${storage_class}" --namespace="${namespace}"
	return $?
}

# delete cluster role
delete_cr()
{
	local namespace="$1"
	local storage_cluster_role="$2"

	oc adm policy remove-cluster-role-from-user "${storage_cluster_role}" \
		system:serviceaccount:kube-system:persistent-volume-binder --namespace="${namespace}"
	oc delete clusterrole "${storage_cluster_role}" --namespace="${namespace}"
	return $?
}

delete_resources()
{
	local namespace="$1"
	local storage_account="$2"
	local storage_class="$3"
	local storage_cluster_role="$4"
	local pvc="$5"
	local pod="$6"

	echo -n 'deleting pod...'
	delete_pod "${namespace}" "${pod}"
	if [ "$?" = "0" ]; then
		echo ok
	else
		echo fail
	fi

	echo -n 'deleting pvc...'
	delete_pvc "${namespace}" "${pvc}"
	if [ "$?" = "0" ]; then
		echo ok
	else
		echo fail
	fi

	echo -n 'deleting pv...'
	delete_pv "${namespace}" "${pvc}"
	if [ "$?" = "0" ]; then
		echo ok
	else
		echo fail
	fi

	echo -n 'deleting sc...'
	delete_sc "${namespace}" "${storage_class}"
	if [ "$?" = "0" ]; then
		echo ok
	else
		echo fail
	fi

	echo -n 'deleting cr...'
	delete_cr "${namespace}" "${storage_cluster_role}"
	if [ "$?" = "0" ]; then
		echo ok
	else
		echo fail
	fi
}

#######################################
# Create operations
#######################################

# create cluster role
create_cr()
{
	local namespace="$1"
	local storage_cluster_role="$2"

	oc create -f rendered/azure-cluster-role.yaml --namespace="${namespace}"
	oc adm policy add-cluster-role-to-user "${storage_cluster_role}" \
		system:serviceaccount:kube-system:persistent-volume-binder --namespace="${namespace}" 
	return $?
}

# create storage class
create_sc()
{
	local namespace="$1"

	oc create -f rendered/azure-storageclass-azure-file.yaml --namespace="${namespace}"
	return $?
}

# create persistent volume
create_pv()
{
	# Do nothing, we are creating a pvc
	return 0
}

# create persistent volume claim
create_pvc()
{
	local namespace="$1"

	oc create -f rendered/azure-pvc.yaml --namespace="${namespace}"
	return $?
}

# create pod to use persistent volume claim
create_pod()
{
	local namespace="$1"

	oc create -f rendered/azure-pvc-pod.yaml --namespace="${namespace}"
	return $?
}

create_resources()
{
	local namespace="$1"
	local storage_account="$2"
	local storage_class="$3"
	local storage_cluster_role="$4"
	local pvc="$5"
	local pod="$6"

	echo -n 'creating cr...'
	create_cr "${namespace}" "${storage_cluster_role}"
	if [ "$?" = "0" ]; then
		echo ok
	else
		echo fail
	fi

	echo -n 'creating sc...'
	create_sc "${namespace}"
	if [ "$?" = "0" ]; then
		echo ok
	else
		echo fail
	fi

	echo -n 'creating pv...'
	create_pv
	if [ "$?" = "0" ]; then
		echo ok
	else
		echo fail
	fi

	echo -n 'creating pvc...'
	create_pvc "${namespace}"
	if [ "$?" = "0" ]; then
		echo ok
	else
		echo fail
	fi

	echo -n 'creating pod...'
	create_pod "${namespace}"
	if [ "$?" = "0" ]; then
		echo ok
	else
		echo fail
	fi
}

#######################################
# Debug functions
#######################################

debug_header()
{
	local msg="$1"

	echo "#"
	echo "# ${msg}"
	echo "#"
}

debug_footer()
{
	echo
}

# debug cluster role
debug_cr()
{
	local namespace="$1"
	local storage_cluster_role="$2"

	debug_header "clusterrole"
	oc get clusterrole "${storage_cluster_role}" --namespace="${namespace}"
	oc describe clusterrole "${storage_cluster_role}" --namespace="${namespace}"
	debug_footer
}

# debug cluster role binding
debug_crb()
{
	local namespace="$1"
	local storage_cluster_role="$2"

	debug_header "clusterrolebinding"
	for crb in $(oc get clusterrolebinding --namespace="${namespace}"|grep ${storage_cluster_role}|grep -v NAME|awk '{ print $1 }'); do
		oc get clusterrolebinding "${crb}"
		oc describe clusterrolebinding "${crb}"
	done
	debug_footer

	return 0
}

# debug storage class
debug_sc()
{
	local namespace="$1"
	local storage_class="$2"

	debug_header "storageclass"
	oc get storageclass "${storage_class}" --namespace="${namespace}"
	oc describe storageclass "${storage_class}" --namespace="${namespace}"
	debug_footer
}

# debug persistent volume
debug_pv()
{
	local namespace="$1"

	debug_header "persistentvolume"
	oc get pv --namespace="${namespace}"
	oc describe pv --namespace="${namespace}"
	debug_footer
}

# debug persistent volume claim
debug_pvc()
{
	local namespace="$1"

	debug_header "persistentvolumeclaim"
	oc get pvc --namespace="${namespace}"
	oc describe pvc --namespace="${namespace}"
	debug_footer
}

# debug pods
debug_pod()
{
	local namespace="$1"
	local pod="$2"

	debug_header "pods"
	oc get pod "${pod}" --namespace="${namespace}"
	oc describe pod "${pod}" --namespace="${namespace}"
	debug_footer
}

# get events
debug_events()
{
	local namespace="$1"

	debug_header "events"
	oc get events --namespace="${namespace}"
	debug_footer
}

debug_resources()
{
	local namespace="$1"
	local storage_account="$2"
	local storage_class="$3"
	local storage_cluster_role="$4"
	local pvc="$5"
	local pod="$6"

	debug_cr "${namespace}" "${storage_cluster_role}"
	debug_crb "${namespace}" "${storage_cluster_role}"
	debug_sc "${namespace}" "${storage_class}"
	debug_pv "${namespace}"
	debug_pvc "${namespace}"
	debug_pod "${namespace}" "${pod}"
	debug_events "${namespace}"
}

main()
{
	local cmd=$1
	case "${cmd}" in
		create|debug|destroy|reset) ;;
		*) usage; exit 1 ;;
	esac

	shift
	[[ -z "$@" ]] && usage && exit 1

	local opt storage_account storage_class storage_cluster_role namespace pvc verbose
	while getopts "a:c:r:n:v" opt; do
		case "${opt}" in
			a) storage_account="${OPTARG}" ;;
			c) storage_class="${OPTARG}" ;;
			r) storage_cluster_role="${OPTARG}" ;;
			n) namespace="${OPTARG}" ;;
			v) verbose=true ;;
			*) usage; exit 1 ;;
		esac
	done
	
	[[ -n "${verbose}" ]] && set -x
	[[ -z "${storage_account}" ]] && usage && exit 1
	[[ -z "${storage_class}" ]] && storage_class="azurerwx"
	[[ -z "${storage_cluster_role}" ]] && storage_cluster_role="azure-secret-reader"
	[[ -z "${namespace}" ]] && namespace="default"
	[[ -z "${pvc}" ]] && pvc="my-debug-pvc"
	[[ -z "${pod}" ]] && pod="my-debug-pod"

	echo "Using the following configuration:"
	echo "-----------------------------------------------"
	echo "namespace:            ${namespace}"
	echo "storage_account:      ${storage_account}"
	echo "storage_class:        ${storage_class}"
	echo "storage_cluster_role: ${storage_cluster_role}"
	echo "-----------------------------------------------"

	[[ -z "${KUBECONFIG}" ]] && echo "KUBECONFIG is not set" && exit 1

	local args="${namespace} ${storage_account} ${storage_class} ${storage_cluster_role} ${pvc} ${pod}"
	fill_templates ${args}

	case "${cmd}" in 
		create)
			create_resources ${args}
			;;
		debug)
			debug_resources ${args}
			;;
		destroy)
			delete_resources ${args}
			;;
		reset)
			delete_resources ${args};
			create_resources ${args}
			;;
		*) echo "Usage: $0 <create|destroy|debug|reset>"; exit 1 ;;
	esac
}

main $*
